# frozen_string_literal: true

class BreadcrumbComponent < BaseComponent
  attr_reader :current_item, :max_visible_items

  def initialize(current_item:, max_visible_items: 3)
    super
    @current_item = current_item
    @max_visible_items = max_visible_items
  end

  def breadcrumb_items
    @breadcrumb_items ||= build_breadcrumb_path
  end

  private

  def build_breadcrumb_path
    items = []
    current = current_item

    while current
      items.unshift(current)
      current = current.parent if current.respond_to?(:parent)
    end

    items
  end

  def truncated?
    breadcrumb_items.size > max_visible_items
  end

  def visible_items
    return breadcrumb_items unless truncated?

    [
      breadcrumb_items.first,
      nil, # represents ellipsis
      *breadcrumb_items.last(max_visible_items - 1)
    ]
  end
end
