import { Given, When, Then } from '@badeball/cypress-cucumber-preprocessor';

let projectId, s1Id, s2Id, s3Id, s4Id;
let tagset1Id, tagset2Id, tagset3Id, tagset4Id;
let doc1Id, doc2Id, doc3Id, doc4Id;

Given('There is a complex directory structure', () => {
  // Create base directory (P1)
  cy.appFactories([
    ['create', 'tagset_directory', 'with_journal_entries', {
      title: 'P1',
      responsible: '1',
      id: 'project1'
    }]
  ]).then(records => {
    projectId = records[0].id;

    // Create S1 directory
    cy.appFactories([
      ['create', 'tagset_directory', {
        title: 'S1',
        responsible: '1',
        parent_tagset_ids: [projectId]
      }]
    ]).then(records => {
      s1Id = records[0].id;

      // Create document 1
      cy.appFactories([
        ['create', 'tagset_doc', 'with_informed', 'with_file', 'with_state', {
          title: 'document 1',
          responsible: '1',
          parent_tagset_ids: [s1Id],
          filename: 'pdf_sample.pdf'
        }]
      ]).then(records => {
        tagset1Id = records[0].id;
        doc1Id = records[0].doc_ids[0];
      });

      // Create S2 directory
      cy.appFactories([
        ['create', 'tagset_directory', {
          title: 'S2',
          responsible: '1',
          parent_tagset_ids: [s1Id]
        }]
      ]).then(records => {
        s2Id = records[0].id;

        // Create document 2
        cy.appFactories([
          ['create', 'tagset_doc', 'with_informed', 'with_file', 'with_state', {
            title: 'document 2',
            responsible: '1',
            parent_tagset_ids: [s2Id],
            filename: 'pdf_sample2.pdf'
          }]
        ]).then(records => {
          tagset2Id = records[0].id;
          doc2Id = records[0].doc_ids[0];

          // Create S3 directory
          cy.appFactories([
            ['create', 'tagset_directory', {
              title: 'S3',
              responsible: '1',
              parent_tagset_ids: [s2Id]
            }]
          ]).then(records => {
            s3Id = records[0].id;

            // Create document 3
            cy.appFactories([
              ['create', 'tagset_doc', 'with_informed', 'with_file', 'with_state', {
                title: 'document 3',
                responsible: '1',
                parent_tagset_ids: [s3Id],
                filename: 'pdf_sample3.pdf'
              }]
            ]).then(records => {
              tagset3Id = records[0].id;
              doc3Id = records[0].doc_ids[0];

              // Create S4 directory
              cy.appFactories([
                ['create', 'tagset_directory', {
                  title: 'S4',
                  responsible: '1',
                  parent_tagset_ids: [s3Id]
                }]
              ]).then(records => {
                s4Id = records[0].id;

                // Create document 4
                cy.appFactories([
                  ['create', 'tagset_doc', 'with_informed', 'with_file', 'with_state', {
                    title: 'document 4',
                    responsible: '1',
                    parent_tagset_ids: [s4Id],
                    filename: 'one_page.pdf'
                  }]
                ]).then(records => {
                  tagset4Id = records[0].id;
                  doc4Id = records[0].doc_ids[0];
                });
              });
            });
          });
        });
      });
    });
  });
});

When('I visit the project root', () => {
  cy.visit(`/spirit_hub/directories/${projectId}/index`);
});

When('I visit the first level folder', () => {
  cy.visit(`/spirit_hub/directories/${s1Id}/index`);
});

When('I visit the first level document', () => {
  cy.visit(`/spirit_hub/tagsets/${tagset1Id}/doc/${doc1Id}`);
});

When('I visit the second level folder', () => {
  cy.visit(`/spirit_hub/directories/${s2Id}/index`);
});

When('I visit the second level document', () => {
  cy.visit(`/spirit_hub/tagsets/${tagset2Id}/doc/${doc2Id}`);
});

When('I visit the third level folder', () => {
  cy.visit(`/spirit_hub/directories/${s3Id}/index`);
});

When('I visit the third level document', () => {
  cy.visit(`/spirit_hub/tagsets/${tagset3Id}/doc/${doc3Id}`);
});

When('I visit the fourth level folder', () => {
  cy.visit(`/spirit_hub/directories/${s4Id}/index`);
});

When('I visit the fourth level document', () => {
  cy.visit(`/spirit_hub/tagsets/${tagset4Id}/doc/${doc4Id}`);
});

Then('I should see {string} in the breadcrumb', (expectedBreadcrumb) => {
    cy.get('[data-controller="breadcrumb"]')
      .invoke('text')
      .then((text) => {
        // Normalize only the > separator, preserving spaces in titles
        const normalizedActual = text.replace(/\s*>\s*/g, ' > ').trim();
        expect(normalizedActual).to.include(expectedBreadcrumb);
      });
  });

  Then('I should see the full path dropdown', () => {
    cy.get('[data-breadcrumb-target="fullPath"]')
      .should('not.have.class', 'hidden');
  });

  Then('the full path should contain {string}', (expectedPath) => {
    cy.get('[data-breadcrumb-target="fullPath"]')
      .invoke('text')
      .then((text) => {
        const normalizedText = text.replace(/\s*>\s*/g, ' > ').trim();
        expect(normalizedText).to.include(expectedPath);
      });
  });

  When('I click on {string} in the dropdown', (linkText) => {
    cy.get('[data-breadcrumb-target="fullPath"]')
      .contains(linkText)
      .click();
  });

  Then('I should be on the S1 directory page', () => {
    cy.url().should('include', `/spirit_hub/directories/${s1Id}/index`);
  });

When('I click on ...', () => {
    cy.get('[data-controller="breadcrumb"]').get('button:contains("...")').click();
});
